require 'rails'
require "neutrino_audiomatic_rails/version"
require "neutrino_audiomatic_rails/neutrino_audiomatic_rails_model"
require 'neutrino_audiomatic_hmac'
require 'active_support/concern'

module Neutrino::AudiomaticRails

  mattr_accessor :parent_controller
  @@parent_controller = "::ApplicationController"

  autoload :'NeutrinoAudiomaticController', 'neutrino_audiomatic_rails/controllers/neutrino_audiomatic_controller'


  @@routes = {}
  
  def self.add_route(klass, strategy)
    @@routes[klass] = {auth_strategy: strategy}
  end

  def  self.gen_callback_path(model, options={})
   url = Rails.application.routes.url_helpers.send( "neutrino_audiomatic_#{ model.class.to_s.underscore }_url")
   case Neutrino::AudiomaticRails.routes[model.class][:auth_strategy]
    when :reference  then
      url
    when :signature then  
      url += "?id=#{model.id}"
      hash = Neutrino::AudiomaticRails::HMAC.gen_hash(url)
      url += "&signature=#{hash}"
    end
  end

  def self.routes
    @@routes
  end  
end

require 'neutrino_audiomatic_rails/rails/routes'
require_relative '../app/jobs/audiomatic_rails/send_to_audiomatic_job'

