require 'open-uri'
require 'tempfile'
module Neutrino::AudiomaticRails
  class NeutrinoAudiomaticController < Neutrino::AudiomaticRails.parent_controller.constantize

    before_action :set_model
    before_action :verify_audiomatic, only: [:update]
    before_action :auth_strategy, only: [:update, :update_with_carrier_wave]

    def set_model 
      klass = request.path[/audiomatic\/(\w+)/, 1]
      @snake_klass = klass.camelize.constantize      
    end

    def download
      model = @snake_klass.to_s
      id = params[:id]
      send_file "#{Rails.root}/tmp/audio/#{model}/#{id}"
    end
   
    def update
      model = find_model
      audiomatic_ans = params[:result]     
      logs = audiomatic_ans.map do |answer|        
        model._save_to answer.to_hash
      end
      logs.each { |log| Rails.logger.error(log) unless log == true }
      model.block_analyze = true
      if model.save
        render  nothing: true, status: 200
      else 
        render  nothing: true, status: 400
      end
    end

  def verify_audiomatic
    unless  Rails.env.test? || Rails.env.development? || Neutrino::AudiomaticRails::HMAC.verify_response(request)
      render nothing: true, status:403 and return
    end 
  end  

  def auth_strategy
    auth = case  Neutrino::AudiomaticRails.routes[@snake_klass][:auth_strategy]
    when :reference then
      reference = params[:ref]
      !@snake_klass.find_by(reference: reference).nil?
    when :signature then
      unless Rails.env.test? or Rails.env.development?
        Neutrino::AudiomaticRails::HMAC.check_hash_url(request)
      else
        true
      end
    end  

    unless auth
      render nothing: true, status:403 and return
    end
  end

  def find_model
    model = case Neutrino::AudiomaticRails.routes[@snake_klass][:auth_strategy]
    when :reference then
      reference = params[:ref]
      @snake_klass.find_by(reference: reference)
    when :signature then
      id = params[:id]
      @snake_klass.find(id)
    end
    render nothing: true, status:400 and return if model.nil?
    model
  end 

  def chose_model(models, mountedas)
    url = params[:url]
    model = models.each { |model| return model if model.send(mountedas.to_sym).url == url }.first
    model
  end  

end
end
